------------------------------------------------------------------------------
--                                                                          --
--                           GNAT RAVENSCAR for NXT                         --
--                                                                          --
--                         Copyright (C) 2012, AdaCore                      --
--                                                                          --
-- This is free software; you can  redistribute it  and/or modify it under  --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. This is distributed in the hope that it will be useful, but WITH-  --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNARL; see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------

with NXT.Filtering;
with NXT.Buttons;     use NXT.Buttons;
with NXT.AVR;
with Ada.Real_Time;   use Ada.Real_Time;

package body NXT.Shutdown is

   Period : constant Time_Span := Milliseconds (50);  -- arbitrary

   Desired_Hold_Time : constant Time_Span := Seconds (3);
   --  The interval in which the user must hold down the Power button for the
   --  shutdown request to be detected

   Samples_Required : constant Positive := Desired_Hold_Time / Period;
   --  The number of successive sampled presses of the power button required
   --  for a forced power off to be recognized

   package Button_Inputs is new NXT.Filtering (Button_Id, No_Button);
   --  Provides noise filtering for button presses

   Forced_Off : Button_Inputs.Filter (Samples_Required);
   --  A filter for detecting a consecutive number of button samples, as
   --  specified by the discriminant. In this case we are not using it to
   --  filter button presses, but rather to simply count them.

   -------------
   -- Monitor --
   -------------

   task body Monitor is
      Next_Time : Time := Clock + Period;
      use Button_Inputs;
   begin
      loop
         --  Check for forced power off by the user
         if Filtered (Forced_Off, Current_Button) = Power_Button then
            loop
               NXT.AVR.Power_Down;
               delay until Clock + Seconds (1);
            end loop;
         end if;

         delay until Next_Time;

         Next_Time := Next_Time + Period;
      end loop;
   end Monitor;

end NXT.Shutdown;
